<?php


class	ClassProCustomer extends ObjectModel
{
	public 		$id_customer;
	
	public		$is_approved;
	public		$privilege_request;
	public		$company;
	public		$privilege_message;
	public		$send_email_approved;
	public		$send_email_request;
	
	public		$field;
	public		$siret;
	
	/** @var string Object creation date */
	public 		$date_add;

	/** @var string Object last modification date */
	public 		$date_upd;
	

	protected 	$fieldsRequired = array('id_customer', 'is_approved');
	protected 	$fieldsValidate = array('id_customer' => 'isUnsignedId',
	'is_approved' => 'isBool',
	'privilege_request' => 'isInt',
	'company' => 'isCleanHtml',
	'send_email_approved' => 'isInt',
	'send_email_request' => 'isInt',
	'privilege_message' => 'isCleanHtml');
	
	protected 	$table = 'pro_customer';
	
	protected $identifier = 'id_customer';

	public function __construct ($id_cust)
	{
		$this->id_customer = $id_cust;
		$this->is_approved = 0;
		$this->privilege_request = 0;
		$this->company = '';
		$this->privilege_message = '';
		$this->send_email_approved = 0;
		$this->send_email_request = 0;
		$this->number_type = 1;
		// parent constructor to load from DB is present
		parent::__construct($id_cust);		
	}
	
	public function getFields()
	{
		parent::validateFields();
		$fields['id_customer'] = intVal($this->id_customer);
		$fields['is_approved'] = $this->is_approved;
		$fields['privilege_request'] = $this->privilege_request;
		$fields['company'] = $this->company;
		$fields['privilege_message'] = $this->privilege_message;
		$fields['send_email_approved'] = $this->send_email_approved;
		$fields['send_email_request'] = $this->send_email_request;
		$fields['number_type'] = $this->number_type;
		$fields['date_add'] = $this->date_add;
		$fields['date_upd'] = $this->date_upd;
		return $fields;
	}
	
	
	static public function isApproved($id_customer)
	{
		// If not existing in this DB, customer is considered as approved
		$dbReturn = Db::getInstance()->ExecuteS('
			SELECT *
			FROM `'._DB_PREFIX_.'pro_customer` ac
			WHERE ac.`id_customer` = '.intval($id_customer). ' and ac.`is_approved` = 0');
		
		return ! (sizeof($dbReturn) > 0);
		
	}
	
	static public function getPrivilegeRequest($id_customer)
	{
		$dbReturn = Db::getInstance()->getRow('
			SELECT ac.*
			FROM `'._DB_PREFIX_.'pro_customer` ac
			WHERE ac.`id_customer` = '.intval($id_customer));
		return $dbReturn;
		
	}
	
	static public function getAllNotApproved()
	{
		$dbReturn = Db::getInstance()->ExecuteS('
			SELECT *
			FROM `'._DB_PREFIX_.'pro_customer` ac
			WHERE  ac.`is_approved` = 0');
		return $dbReturn;
		
	}
	
	static public function getAllNotProAccess()
	{
		$dbReturn = Db::getInstance()->ExecuteS('
			SELECT *
			FROM `'._DB_PREFIX_.'pro_customer` ac
			WHERE  ac.`privilege_request` = 1');
		return $dbReturn;
		
	}	
	
	static public function onlyNumeric($field)
	{
		$character = 0; 
		$size = strlen($field); 

		if (preg_match("/[0-9]/i", $field)) { // We test if the string contains digits 
			while ($character <= $size-1) {	// so that there are characters 
				$Chain = $field{$character}; // Taking character by character 
				if((ord($Chain) <= ord("9")) && (ord($Chain) >= ord("0")) ){ // if the character is a digit 
					$number .= $Chain; 
				} 
				$character++; 
			} 
		} 
		return $number;
	}
	
	// function to check the validity of a SIRET
	static public function isSiret($siret)
	{
		if (strlen($siret) != 14) return 1; // the SIRET must contain 14 characters
		if (!is_numeric($siret)) return 2; // the SIRET must contain only numbers

		// we take each item one by one
		// if its index (position in the chain starts at 0 at first character) is even
		// we double its value if it exceeds 9, it subtracts 9
		// we add this value to the total


		for ($index = 0; $index < 14; $index ++)
		{
			$number = (int) $siret[$index];
			if (($index % 2) == 0) { if (($number *= 2) > 9) $number -= 9; }
			$sum += $number;
		}

		// the number is valid if the sum of digits is multiple of 10
		if (($sum % 10) != 0) return 3; else return 0;		
	}	
	
	static public function isSiren($siren)
	{
		if (strlen($siren) != 9) return 1; // le SIREN doit contenir 9 caractres
		if (!is_numeric($siren)) return 2; // le SIREN ne doit contenir que des chiffres

		// on prend chaque chiffre un par un
		// si son index (position dans la chane en commence  0 au premier caractre) est impair
		// on double sa valeur et si cette dernire est suprieure  9, on lui retranche 9
		// on ajoute cette valeur  la somme totale

		for ($index = 0; $index < 9; $index ++)
		{
			$number = (int) $siren[$index];
			if (($index % 2) != 0) { if (($number *= 2) > 9) $number -= 9; }
			$sum += $number;
		}

		// le numro est valide si la somme des chiffres est multiple de 10
		if (($sum % 10) != 0) return 3; else return 0;		
	}

#
# Checks for any valid European Union value added tax number
# from original Cobol public version
# http://a104.g.akamai.net/7/104/3242/v001/www.aeat.es/padres/prevalid/descarga/prv34902.exe
# to PHP by David Gimeno i Ayuso, info@sima.cat. Catalonia 2002.
#
# input  $vat          vat number (upper or lowercase, separators included)
#        $country      valid ISO country code of any EU-15
#                      note: GR is the ISO country code for Greece but EU
#                      directive 2001/115 changed it to EL
#
# output $vat          formatted vat number (uppercase, without separators)
#        function      false     valid
#                      1         invalid format
#                      2         invalid value
#                      3         invalid check
#                      4         invalid country
#----------------------------------------------------------------------------

	static public function isVatNumber($data) {

		$value   = preg_replace("/[^A-Z0-9]/","", strtoupper($data));
		$COUNTRY = substr($value,0,2);
		$VAT     = substr($value, 2,strlen($value)-2);	

		switch (strtoupper($COUNTRY)):
	case 'AT':
		#
		# Austria
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^U{1})([0-9]{1,7})([0-9]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                         //invalid format
		endif;
		$TIP=strtoupper($REGS[1]);
		$NUM=$REGS[2];
		$CTR=$REGS[3];
		$NUM=str_pad($NUM,7,'0',STR_PAD_LEFT);
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I++):
		$C=substr($NUM,$I,1)*($I%2==0?1:2);
		$CHK=$CHK+substr($C,0,1)+substr($C,1,1);
		endfor;
		$CHK=10-($CHK+4)%10;
		$CHK=($CHK==10?0:$CHK);
		if (!preg_match('/\?{1}/',$CTR) and $CTR<>$CHK):
		return 3;                         //invalid check
		endif;
		$VAT=$TIP.$NUM.$CHK;
		return 0;
	case 'BE':
		#
		# Belgium
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{1,8})([0-9]{2}$|\?{1,2}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                         //invalid format
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		$NUM=str_pad($NUM,8,'0',STR_PAD_LEFT);
		if (substr($NUM,1,1)<2):
		return 2;                         //invalid value
		endif;
		$CHK=str_pad(97-$NUM%97,2,'0',STR_PAD_LEFT);
		if (!preg_match('/\?{1,2}/',$CTR) and $CTR<>$CHK):
		return 3;                         //invalid check
		endif;
		$VAT=$NUM.$CHK;
		return 0;
	case 'BG':
		#
		# Bulgaria
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{1,9})([0-9]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                         //invalid format
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		$NUM=str_pad($NUM,9,'0',STR_PAD_LEFT);
		if (preg_match('/[23]/',substr($NUM,0,1)) and substr($NUM,1,2)<>'22'):
		return 2;                         //invalid value
		endif;
		$CHG=array(4,3,2,7,6,5,4,3,2);
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I++):
		$CHK=$CHK+$CHG[$I]*substr($NUM,$I,1);
		endfor;
		$CHK=11-$CHK%11;
		$CHK=($CHK==11?0:$CHK);
		if ($CHK==10):
		return 2;                         //invalid value
		endif;
		if (!preg_match('/\?{1}/',$CTR) and $CTR<>$CHK):
		return 3;                         //invalid check
		endif;
		$VAT=$NUM.$CHK;
		return 0;
	case 'CY':
		#
		# Cyprus
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{1,8})([A-Z]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                         //invalid format
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		$NUM=str_pad($NUM,8,'0',STR_PAD_LEFT);
		if (!preg_match('/[013459]/',substr($NUM,0,1))):
		return 2;                         //invalid value
		endif;
		$CHG=array(1,0,5,7,9,13,15,17,19,21);
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I++):
		if ($I%2==0):
		$CHK=$CHK+$CHG[substr($NUM,$I,1)];
else:
		$CHK=$CHK+substr($NUM,$I,1);
		endif;
		endfor;
		$CHK=substr('ABCDEFGHIJKLMNOPQRSTUVWXYZ',$CHK%26,1);
		if (!preg_match('/\?{1}/',$CTR) and $CTR<>$CHK):
		return 3;                         //invalid check
		endif;
		$VAT=$NUM.$CHK;
		return 0;
	case 'CZ':
		#
		# Czech Republic
		#
		$FAN=strtoupper(preg_replace('[^[:alnum:]\?]','',$VAT));
		$OK=preg_match('/(^[012345678][0-9]{6})([0-9]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		$OK=preg_match('/(^[6][0-9]{7})([0-9]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		$OK=preg_match('/(^[0-9]{9}$)/',$FAN,$REGS);
		if (!$OK):
		$OK=preg_match('/(^[0-9]{10}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                   //invalid format
		endif;
		endif;
		endif;
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		if (strlen($NUM)>8):
		if (strlen($NUM)==9):
		if (substr($NUM,0,2)>53 and substr($NUM,0,2)<80):
		return 2;                     //invalid value
		endif;
else:
		if (substr($NUM,0,2)<54):
		return 2;                     //invalid value
		endif;
		endif;
		if (substr($NUM,2,2)<1 or substr($NUM,2,2)>12 and substr($NUM,2,2)<51 or substr($NUM,2,2)>62):
		return 2;                       //invalid value
		endif;
		if (substr($NUM,4,2)<1):
		return 2;                       //invalid value
		endif;
		if (preg_match('/02|52/',substr($NUM,2,2))):
		if (substr($NUM,0,2)%4>0):
		if (substr($NUM,4,2)>28):
		return 2;                   //invalid value
		endif;
else:
		if (substr($NUM,4,2)>29):
		return 2;                   //invalid value
		endif;
		endif;
		elseif (preg_match('/04|06|09|11|54|56|59|61/',substr($NUM,2,2))):
		if (substr($NUM,4,2)>30):
		return 2;                     //invalid value
		endif;
else:
		if (substr($NUM,4,2)>31):
		return 2;                     //invalid value
		endif;
		endif;
		endif;
		if (strlen($NUM)<9):
		$CHG=array(8,7,6,5,4,3,2);
		$CHK=0;
		for ($I=0; $I<7; $I++):
		$CHK=$CHK+$CHG[$I]*substr($NUM,$I+(1 and strlen($NUM)==8),1);
		endfor;
		if (strlen($NUM)==7):
		$CHK=11-$CHK%11;
		$CHK=($CHK==10?0:$CHK);
		$CHK=($CHK==11?1:$CHK);
else:
		$CHK=9-(11-$CHK%11)%10;
		endif;
		if (!preg_match('/\?{1}/',$CTR) and $CTR<>$CHK):
		return 3;                       //invalid check
		endif;
		$VAT=$NUM.$CHK;
		elseif (strlen($NUM)==9):
		$VAT=$NUM;
else:
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I=$I+2):
		$CHK=$CHK+substr($NUM,$I,2);
		endfor;
		if ($CHK%11>0 or fmod($NUM,11)>0):
		return 3;                       //invalid check
		endif;
		$VAT=$NUM;
		endif;
		return 0;
	case 'DE':
		#
		# Germany
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{1,8})([0-9]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                         //invalid format
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		if ($NUM==0):
		return 2;                         //invalid value
		endif;
		$NUM=str_pad($NUM,8,'0',STR_PAD_LEFT);
		//
		// original formula for Germany
		//
		//$CHK=10;
		//for ($I=0; $I<strlen($NUM); $I++):
		//  $CHK=($CHK+substr($NUM,$I,1))%10; $CHK=($CHK==0?10:$CHK);
		//  $CHK=2*$CHK%11; $CHK=($CHK==0?11:$CHK);
		//endfor;
		//$CHK=11-$CHK; $CHK=($CHK==10?0:$CHK);
		//
		// optimized formula follows
		//
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I++):
		$CHK=2*((substr($NUM,$I,1)+$CHK+9)%10+1)%11;
		endfor;
		$CHK=11-$CHK; $CHK=($CHK==10?0:$CHK);
		if (!preg_match('\?{1}',$CTR) and $CTR<>$CHK):
		return 3;                         //invalid check
		endif;
		$VAT=$NUM.$CHK;
		return 0;
	case 'DK':
		#
		# Denmark
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]]/','',$VAT));
		$OK=preg_match('/(^[0-9]{1,8})/',$FAN,$REGS);
		if (!$OK):
		return 1;                         //invalid format
		endif;
		$NUM=$REGS[1];
		if (substr($NUM,0,1)<1):
		return 2;                         //invalid value
		endif;
		$NUM=substr($NUM,0,1).str_pad(substr($NUM,1),7,'0',STR_PAD_LEFT);
		$CHK=2*substr($NUM,0,1);
		for ($I=1; $I<strlen($NUM); $I++):
		$CHK=$CHK+(8-$I)*substr($NUM,$I,1);
		endfor;
		if ($CHK%11>0):
		return 3;                         //invalid check
		endif;
		$VAT=$NUM;
		return 0;
	case 'EE':
		#
		# Estonia
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{1,8})([0-9]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                         //invalid format
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		$NUM=str_pad($NUM,8,'0',STR_PAD_LEFT);
		$CHG=array(3,7,1,3,7,1,3,7);
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I++):
		$CHK=$CHK+$CHG[$I]*substr($NUM,$I,1);
		endfor;
		$CHK=10-$CHK%10;
		$CHK=($CHK==10?0:$CHK);
		if (!preg_match('\?{1}',$CTR) and $CTR<>$CHK):
		return 3;                         //invalid check
		endif;
		$VAT=$NUM.$CHK;
		return 0;
	case 'ES':
		#
		# Spain
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[A-Z]{1})([0-9]{1,7})([0-9]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		$OK=preg_match('/(^[A-Z]{1})([0-9]{1,7})([A-Z]{1}$)/',$FAN,$REGS);
		if (!$OK):
		$OK=preg_match('/()(^[0-9]{1,8})([A-Z]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                     //invalid format
		endif;
		endif;
		endif;
		$TIP=$REGS[1];
		$NUM=$REGS[2];
		$CTR=$REGS[3];
		if ($TIP==''):
		$NUM=str_pad($NUM,8,'0',STR_PAD_LEFT);
else:
		$NUM=str_pad($NUM,7,'0',STR_PAD_LEFT);
		endif;
		if (preg_match('[A-H]|[JNPQRSUVW]',$TIP)):
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I++):
		$C=substr($NUM,$I,1)*($I%2>0?1:2);
		$CHK=$CHK+substr($C,0,1)+substr($C,1,1);
		endfor;
		$CHK=10-$CHK%10;
		$CHK=($CHK==10?0:$CHK);
		if (preg_match('[NPQRSW]',$TIP)):
		$CHK=substr('JABCDEFGHI',$CHK,1);
		endif;
		elseif (preg_match('[KLMX]',$TIP) or ($TIP=='')):
		$CHK=$NUM%23;
		$CHK=substr('TRWAGMYFPDXBNJZSQVHLCKE',$CHK,1);
		elseif (preg_match('[Y]',$TIP)):
		$CHK=(10000000+$NUM)%23;
		$CHK=substr('TRWAGMYFPDXBNJZSQVHLCKE',$CHK,1);
		elseif (preg_match('[Z]',$TIP)):
		$CHK=(20000000+$NUM)%23;
		$CHK=substr('TRWAGMYFPDXBNJZSQVHLCKE',$CHK,1);
else:
		return 2;                         //invalid value
		endif;
		if (!preg_match('\?{1}',$CTR) and $CTR<>$CHK):
		return 3;                         //invalid check
		endif;
		$VAT=$TIP.$NUM.$CHK;
		return 0;
	case 'FI':
		#
		# Finland
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{1,7})([0-9]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                         //invalid format
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		$NUM=str_pad($NUM,7,'0',STR_PAD_LEFT);
		$CHG=array(7,9,10,5,8,4,2);
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I++):
		$CHK=$CHK+$CHG[$I]*substr($NUM,$I,1);
		endfor;
		$CHK=11-$CHK%11;
		$CHK=($CHK==11?0:$CHK);
		if ($CHK==10):
		return 2;                         //invalid value
		endif;
		if (!preg_match('/\?{1}/',$CTR) and $CTR<>$CHK):
		return 3;                         //invalid check
		endif;
		$VAT=$NUM.$CHK;
		return 0;
	case 'FR':
		#
		# France
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{2}|^\?{1,2})([0-9]{1,9}$)/',$FAN,$REGS);
		if (!$OK):
		$OK=preg_match('/(^[[:alnum:]]{2})([0-9]{1,9}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                       //invalid format
		endif;
		endif;
		$CTR=$REGS[1];
		$NUM=$REGS[2];
		$NUM=str_pad($NUM,9,'0',STR_PAD_LEFT);
		if ($NUM==0):
		return 2;                         //invalid value
		elseif (preg_match('/I|O/',$CTR)):
		return 2;                         //invalid value
		elseif (is_numeric($CTR) or preg_match('\?{1,2}',$CTR)):
		$CHK=str_pad((($NUM%97)*100+12)%97,2,'0',STR_PAD_LEFT);
		if (!preg_match('/\?{1,2}/',$CTR) and $CTR<>$CHK):
		return 3;                       //invalid check
		endif;
		elseif (preg_match('/[A-Z]/',substr($CTR,0,1))==preg_match('/[A-Z]/',substr($CTR,1,1))):
		return 2;                         //invalid value
else:
		$CHG=array_merge(range('0','9'),range('A','H'),range('J','N'),range('P','Z'));
		$C=array_search(substr($CTR,0,1),$CHG);
		$CHK=($C<10?$C*24-10:$C*34-100)+array_search(substr($CTR,1,1),$CHG);
		$C=$CHK%11;
		$CHK=floor($CHK/11)+1;
		if ($C<>($NUM+$CHK)%11):
		return 3;                       //invalid check
		endif;
		$CHK=$CTR;
		endif;
		$VAT=$CHK.$NUM;
		return 0;
	case 'GB':
		#
		# United Kingdom
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]]/','',$VAT));
		$OK=preg_match('/(^000|^001)([0-9]{9})([3]{1}$)/',$FAN,$REGS);
		if (!$OK):
		$OK=preg_match('/(^000|^001)([0-9]{9}$)()/',$FAN,$REGS);
		if (!$OK):
		$OK=preg_match('/()(^[0-9]{9})([3]{1}$)/',$FAN,$REGS);
		if (!$OK):
		$OK=preg_match('/()(^[0-9]{9}$)()/',$FAN,$REGS);
		if (!$OK):
		$OK=preg_match('/(^HA)([0-9]{3}$)()/',$FAN,$REGS);
		if (!$OK):
		$OK=preg_match('/(^GD)([0-9]{3}$)()/',$FAN,$REGS);
		if (!$OK):
		return 1;               //invalid format
		endif;
		endif;
		endif;
		endif;
		endif;
		endif;
		$TIP=$REGS[1];
		$NUM=$REGS[2];
		$CTR=$REGS[3];
		if (is_string($TIP)):
		if ($TIP=='GD' and $NUM>499 or $TIP=='HA' and $NUM<500):
		return 3;                       //invalid check
		endif;
else:
		if ($TIP>1):
		return 3;                       //invalid check
		endif;
		if (substr($NUM,0,7)<1 or substr($NUM,0,7)>19999 and substr($NUM,0,7)<1000000):
		return 3;                       //invalid check
		endif;
		if (substr($NUM,7)>=97):
		return 3;                       //invalid check
		endif;
		if ($CTR>'' and $CTR<>3):
		return 3;                       //invalid check
		endif;
		$CHK=0;
		for ($I=0; $I<strlen($NUM)-2; $I++):
		$CHK=$CHK+(8-$I)*substr($NUM,$I,1);
		endfor;
		$CHK=$CHK+substr($NUM,-2);
		if ($CHK%97<>0):
		return 3;                       //invalid check
		endif;
		endif;
		$VAT=$TIP.$NUM.$CTR;
		return 0;
	case 'GR': case 'EL':
		#
		# Greece
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{8})([0-9]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		$OK=preg_match('/(^[0-9]{7})([0-9]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                       //invalid format
		endif;
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I++):
		$CHK=$CHK+(pow(2,strlen($NUM)-$I))*substr($NUM,$I,1);
		endfor;
		$CHK=$CHK%11;
		$CHK=($CHK==10?0:$CHK);
		if (!preg_match('/\?{1}/',$CTR) and $CTR<>$CHK):
		return 3;                         //invalid check
		endif;
		$VAT=$NUM.$CHK;
		return 0;
	case 'HU':
		#
		# Hungary
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{1,7})([0-9]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                         //invalid format
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		$NUM=str_pad($NUM,7,'0',STR_PAD_LEFT);
		if (substr($NUM,0,1)==0):
		return 2;                         //invalid value
		endif;
		$CHG=array(9,7,3,1,9,7,3);
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I++):
		$CHK=$CHK+$CHG[$I]*substr($NUM,$I,1);
		endfor;
		$CHK=10-$CHK%10;
		$CHK=($CHK==10?0:$CHK);
		if (!preg_match('/\?{1}/',$CTR) and $CTR<>$CHK):
		return 3;                         //invalid check
		endif;
		$VAT=$NUM.$CHK;
		return 0;
	case 'IE':
		#
		# Ireland
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\*\+\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{1})([A-Z\*\+]{1})([0-9]{1,5})([A-Z\?]{1}$)/',$FAN,$REGS);
		if (!$OK):
		$OK=preg_match('/()()(^[0-9]{1,7})([A-Z]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                       //invalid format
		endif;
		endif;
		$NUM=$REGS[1].$REGS[3];
		$TIP=$REGS[2];
		$CTR=$REGS[4];
		if ($TIP>''):
		if (substr($NUM,0,1)<7):
		return 2;                       //invalid value
		endif;
		$NUM=substr($NUM,0,1).str_pad(substr($NUM,1),5,'0',STR_PAD_LEFT);
		$CHK=2*substr($NUM,0,1);
		for ($I=1; $I<strlen($NUM); $I++):
		$CHK=$CHK+(8-$I)*substr($NUM,$I,1);
		endfor;
else:
		if ($NUM==0):
		return 2;                       //invalid value
		endif;
		$NUM=str_pad($NUM,7,'0',STR_PAD_LEFT);
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I++):
		$CHK=$CHK+(8-$I)*substr($NUM,$I,1);
		endfor;
		endif;
		$CHK=$CHK%23;
		$CHK=substr('WABCDEFGHIJKLMNOPQRSTUV',$CHK,1);
		if (!preg_match('/\?{1}/',$CTR) and $CTR<>$CHK):
		return 3;                         //invalid check
		endif;
		$VAT=substr($NUM,0,1).$TIP.substr($NUM,1).$CHK;
		return 0;
	case 'IT':
		#
		# Italy
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{1,10})([0-9]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                         //invalid format
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		$NUM=str_pad($NUM,10,'0',STR_PAD_LEFT);
		if (substr($NUM,0,7)==0):
		return 2;                         //invalid value
		endif;
		if (substr($NUM,7)<1 or substr($NUM,7)>100 and substr($NUM,7)<>120 and substr($NUM,7)<>121):
		return 2;                         //invalid value
		endif;
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I++):
		$C=substr($NUM,$I,1)*($I%2==0?1:2);
		$CHK=$CHK+substr($C,0,1)+substr($C,1,1);
		endfor;
		$CHK=10-$CHK%10;
		$CHK=($CHK==10?0:$CHK);
		if (!preg_match('/\?{1}/',$CTR) and $CTR<>$CHK):
		return 3;                         //invalid check
		endif;
		$VAT=$NUM.$CHK;
		return 0;
	case 'LT':
		#
		# Lithuania
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{11})([0-9]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		$OK=preg_match('/(^[0-9]{8})([0-9]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                       //invalid format
		endif;
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		if (substr($NUM,-1,1)<>1):
		return 2;                         //invalid value
		endif;
		$CHG=array(1,2,3,4,5,6,7,8,9,1,2,3,4);
		do {
			$CHK=0;
			for ($I=0; $I<strlen($NUM); $I++):
			$CHK=$CHK+$CHG[$I]*substr($NUM,$I,1);
			endfor;
			if (count($CHG)<13):
			break;
			endif;
			array_shift($CHG); array_shift($CHG);
		} while ($CHK%11==10);
		$CHK=$CHK%11;
		$CHK=($CHK==10?0:$CHK);
		if (!preg_match('/\?{1}/',$CTR) and $CTR<>$CHK):
		return 3;                         //invalid check
		endif;
		$VAT=$NUM.$CHK;
		return 0;
	case 'LU':
		#
		# Luxemburg
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{1,6})([0-9]{2}$|\?{1,2}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                         //invalid format
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		if ($NUM==0):
		return 2;                         //invalid value
		endif;
		$NUM=str_pad($NUM,6,'0',STR_PAD_LEFT);
		$CHK=str_pad($NUM%89,2,'0',STR_PAD_LEFT);
		if (!preg_match('/\?{1,2}/',$CTR) and $CTR<>$CHK):
		return 3;                         //invalid check
		endif;
		$VAT=$NUM.$CHK;
		return 0;
	case 'LV':
		#
		# Latvia
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[4-9][0-9]{9})([0-9]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		$OK=preg_match('/(^[0-3][0-9]{10})/',$FAN,$REGS);
		if (!$OK):
		return 1;                       //invalid format
		endif;
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		if (substr($NUM,0,1)<4):
		if (substr($NUM,0,2)<1):
		return 2;                       //invalid value
		endif;
		if (preg_match('/02/',substr($NUM,2,2))):
		if (substr($NUM,4,2)%4>0):
		if (substr($NUM,0,2)>28):
		return 2;                   //invalid value
		endif;
else:
		if (substr($NUM,0,2)>29):
		return 2;                   //invalid value
		endif;
		endif;
		elseif (preg_match('/04|06|09|11/',substr($NUM,2,2))):
		if (substr($NUM,0,2)>30):
		return 2;                     //invalid value
		endif;
else:
		if (substr($NUM,0,2)>31):
		return 2;                     //invalid value
		endif;
		endif;
		if (substr($NUM,2,2)<1 or substr($NUM,2,2)>12):
		return 2;                       //invalid value
		endif;
		endif;
		if (substr($NUM,0,1)>3):
		$CHG=array(9,1,4,8,3,10,2,5,7,6);
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I++):
		$CHK=$CHK+$CHG[$I]*substr($NUM,$I,1);
		endfor;
		if ($CHK%11==4 and substr($NUM,0,1)==9):
		$CHK=$CHK-45;
		endif;
		if ($CHK%11==4):
		$CHK=4-$CHK%11;
		elseif ($CHK%11>4):
		$CHK=14-$CHK%11;
else:
		$CHK=3-$CHK%11;
		endif;
		if (!preg_match('/\?{1}/',$CTR) and $CTR<>$CHK):
		return 3;                       //invalid check
		endif;
		endif;
		$VAT=$NUM.$CHK;
		return 0;
	case 'MT':
		#
		# Malta
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{1,6})([0-9]{2}$|\?{1,2}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                         //invalid format
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		$NUM=str_pad($NUM,6,'0',STR_PAD_LEFT);
		if ($NUM<100001):
		return 2;                         //invalid value
		endif;
		$CHG=array(3,4,6,7,8,9);
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I++):
		$CHK=$CHK+$CHG[$I]*substr($NUM,$I,1);
		endfor;
		$CHK=str_pad(37-$CHK%37,2,'0',STR_PAD_LEFT);
		if (!preg_match('/\?{1,2}/',$CTR) and $CTR<>$CHK):
		return 3;                         //invalid check
		endif;
		$VAT=$NUM.$CHK;
		return 0;
	case 'NL':
		#
		# Netherlands
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{1,8})([0-9]{1}|\?{1})([B]{1})([0-9]{2}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                         //invalid format
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		$TIP=$REGS[3].$REGS[4];
		if ($NUM==0):
		return 2;                         //invalid value
		endif;
		$NUM=str_pad($NUM,8,'0',STR_PAD_LEFT);
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I++):
		$CHK=$CHK+(9-$I)*substr($NUM,$I,1);
		endfor;
		$CHK=$CHK%11;
		if ($CHK==10):
		return 2;                         //invalid value
		endif;
		if (!preg_match('/\?{1}/',$CTR) and $CTR<>$CHK):
		return 3;                         //invalid check
		endif;
		$VAT=$NUM.$CHK.$TIP;
		return 0;
	case 'PL':
		#
		# Poland
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{1,9})([0-9]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                         //invalid format
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		$NUM=str_pad($NUM,9,'0',STR_PAD_LEFT);
		$CHG=array(6,5,7,2,3,4,5,6,7);
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I++):
		$CHK=$CHK+$CHG[$I]*substr($NUM,$I,1);
		endfor;
		$CHK=$CHK%11;
		if ($CHK==10):
		return 2;                         //invalid value
		endif;
		if (!preg_match('/\?{1}/',$CTR) and $CTR<>$CHK):
		return 3;                         //invalid check
		endif;
		$VAT=$NUM.$CHK;
		return 0;
	case 'PT':
		#
		# Portugal
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{1,8})([0-9]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                         //invalid format
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		if (substr($NUM,0,1)<'1'):
		return 2;                         //invalid value
		endif;
		$NUM=substr($NUM,0,1).str_pad(substr($NUM,1),7,'0',STR_PAD_LEFT);
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I++):
		$CHK=$CHK+(9-$I)*substr($NUM,$I,1);
		endfor;
		$CHK=11-$CHK%11;
		$CHK=($CHK>9?0:$CHK);
		if (!preg_match('/\?{1}/',$CTR) and $CTR<>$CHK):
		return 3;                         //invalid check
		endif;
		$VAT=$NUM.$CHK;
		return 0;
	case 'RO':
		#
		# Romania
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{1,9})([0-9]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		$OK=preg_match('/(^[12346][0-9]{11})([0-9]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                       //invalid format
		endif;
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		if (strlen($NUM)==12):
		if (substr($NUM,3,2)<1 or substr($NUM,3,2)>12):
		return 2;                       //invalid value
		endif;
		if (preg_match('/02/',substr($NUM,3,2))):
		if (substr($NUM,1,2)%4>0):
		if (substr($NUM,5,2)>28):
		return 2;                   //invalid value
		endif;
else:
		if (substr($NUM,5,2)>29):
		return 2;                   //invalid value
		endif;
		endif;
		elseif (preg_match('/04|06|09|11/',substr($NUM,3,2))):
		if (substr($NUM,5,2)>30):
		return 2;                     //invalid value
		endif;
else:
		if (substr($NUM,5,2)>31):
		return 2;                     //invalid value
		endif;
		endif;
		$CHG=array(2,7,9,1,4,6,3,5,8,2,7,9);
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I++):
		$CHK=$CHK+$CHG[$I]*substr($NUM,$I,1);
		endfor;
		$CHK=$CHK%11;
		$CHK=($CHK==10?1:$CHK);
		if (!preg_match('\?{1}',$CTR) and $CTR<>$CHK):
		return 3;                       //invalid check
		endif;
else:
		$NUM=str_pad($NUM,9,'0',STR_PAD_LEFT);
		$CHG=array(7,5,3,2,1,7,5,3,2);
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I++):
		$CHK=$CHK+$CHG[$I]*substr($NUM,$I,1);
		endfor;
		$CHK=$CHK*10%11;
		$CHK=($CHK==10?0:$CHK);
		if (!preg_match('/\?{1}/',$CTR) and $CTR<>$CHK):
		return 3;                       //invalid check
		endif;
		endif;
		$VAT=$NUM.$CHK;
		return 0;
	case 'SE':
		#
		# Sweden
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{1,9})([0-9]{1}|\?{1})([0-9]{2}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                         //invalid format
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		$TIP=$REGS[3];
		if ($TIP==0):
		return 2;                         //invalid value
		endif;
		$NUM=str_pad($NUM,9,'0',STR_PAD_LEFT);
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I++):
		$C=substr($NUM,$I,1)*($I%2>0?1:2);
		$CHK=$CHK+substr($C,0,1)+substr($C,1,1);
		endfor;
		$CHK=10-$CHK%10;
		$CHK=($CHK==10?0:$CHK);
		if (!preg_match('/\?{1}/',$CTR) and $CTR<>$CHK):
		return 3;                         //invalid check
		endif;
		$VAT=$NUM.$CHK.$TIP;
		return 0;
	case 'SI':
		#
		# Slovenia
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{1,7})([0-9]{1}$|\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                         //invalid format
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		$NUM=str_pad($NUM,7,'0',STR_PAD_LEFT);
		if ($NUM<1000000):
		return 2;                         //invalid value
		endif;
		$CHG=array(8,7,6,5,4,3,2);
		$CHK=0;
		for ($I=0; $I<strlen($NUM); $I++):
		$CHK=$CHK+$CHG[$I]*substr($NUM,$I,1);
		endfor;
		$CHK=11-$CHK%11;
		$CHK=($CHK==10?0:$CHK);
		$CHK=($CHK==11?1:$CHK);
		if (!preg_match('/\?{1}/',$CTR) and $CTR<>$CHK):
		return 3;                         //invalid check
		endif;
		$VAT=$NUM.$CHK;
		return 0;
	case 'SK':
		#
		# Slovakia
		#
		$FAN=strtoupper(preg_replace('/[^[:alnum:]\?]/','',$VAT));
		$OK=preg_match('/(^[0-9]{9})(\?{1}$)/',$FAN,$REGS);
		if (!$OK):
		$OK=preg_match('/(^[0-9]{10}$)/',$FAN,$REGS);
		if (!$OK):
		$OK=preg_match('/(^[0-9]{9}$)/',$FAN,$REGS);
		if (!$OK):
		return 1;                     //invalid format
		endif;
		endif;
		endif;
		$NUM=$REGS[1];
		$CTR=$REGS[2];
		if ($CTR>''):
		if (substr($NUM,0,2)>0):
		return 2;                       //invalid value
		endif;
		elseif (strlen($NUM)==9 or (substr($NUM,0,2)>0 and substr($NUM,0,2)<>10 and substr($NUM,0,2)<>20)):
		if (strlen($NUM)==9):
		if (substr($NUM,0,2)>53):
		return 2;                     //invalid value
		endif;
else:
		if (substr($NUM,0,2)<54):
		return 2;                     //invalid value
		endif;
		endif;
		if (substr($NUM,2,2)<1 or substr($NUM,2,2)>12 and substr($NUM,2,2)<51 or substr($NUM,2,2)>62):
		return 2;                       //invalid value
		endif;
		if (substr($NUM,4,2)<1):
		return 2;                       //invalid value
		endif;
		if (preg_match('/02|52/',substr($NUM,2,2))):
		if (substr($NUM,0,2)%4>0):
		if (substr($NUM,4,2)>28):
		return 2;                   //invalid value
		endif;
else:
		if (substr($NUM,4,2)>29):
		return 2;                   //invalid value
		endif;
		endif;
		elseif (preg_match('/04|06|09|11|54|56|59|61/',substr($NUM,2,2))):
		if (substr($NUM,4,2)>30):
		return 2;                     //invalid value
		endif;
else:
		if (substr($NUM,4,2)>31):
		return 2;                     //invalid value
		endif;
		endif;
		endif;
		if ($CTR>''):
		$CHG=array(8,7,6,5,4,3,2);
		$CHK=0;
		for ($I=2; $I<strlen($NUM); $I++):
		$CHK=$CHK+$CHG[$I]*substr($NUM,$I,1);
		endfor;
		$CHK=11-$CHK%11;
		$CHK=($CHK==10?0:$CHK);
		$CHK=($CHK==11?1:$CHK);
		$VAT=$NUM.$CHK;
else:
		$VAT=$NUM;
		endif;
		return 0;
	default:
		return 4;                           //invalid country
		endswitch;
	}
	
}

?>