<?php

/**
 *
 * Module Pro Customer for PrestaShop
 * Developed by DPFPIC (http://dpfpic.com)
 *
 */

class pro_customer extends Module
{
	const __MAIL_DELIMITOR__ = ',';

	public function __construct()
	{
		$this->name = 'pro_customer';
		$this->tab = 'DPFPIC - Modules';
		$this->version = 1.2; // compatible with PS 1.4.
		$this->author = 'DPFPIC';
		$this->date = '(12/08/2011)';

		parent::__construct(); // The parent construct is required for translations

		$this->page = basename(__FILE__, '.php');
		$this->displayName = $this->l('Pro Customer Management');
		$this->description = $this->l('This module allows to validate pro customer');
		
		$this->_isCustomerRegistredByDefault = intval(Configuration::get('CUSTREG_IS_REGISTERED_BY_DEFAULT'));
		$this->_sendAdminMail = intval(Configuration::get('CUSTREG_SEND_ADMIN_MAIL'));
		$this->_mails =  Configuration::get('CUSTREG_MAILS');
		$this->_defaultGroups =  explode (';', Configuration::get('CUSTREG_GROUPS'));
		$this->_idDefaultGroups = Configuration::get('CUSTREG_ID_GROUPS');
		$this->_registrationCode =  Configuration::get('CUSTREG_REGISTRATION_CODE');
		$this->_askPrivilegeAccess =  Configuration::get('CUSTREG_ASK_PRIVILEGE_ACCESS');
		
		$this->pro_imgname = 'pro_custom.jpg';

		if (!file_exists(dirname(__FILE__).'/img/'.$this->pro_imgname))
		$this->pro_img = _MODULE_DIR_.$this->name.'/img/pro.jpg';
		else
		$this->pro_img = _MODULE_DIR_.$this->name.'/img/'.$this->pro_imgname;
		$this->pro_link = htmlentities(Configuration::get('PRO_CUSTOMER_LINK'), ENT_QUOTES, 'UTF-8');
	}

	public function install()
	{
				if (!parent::install())
		return false;
		if (!$this->registerHook('authentication') or
				!$this->registerHook('createAccountForm') or
				!$this->registerHook('createAccount') or
				!$this->registerHook('backOfficeHome') or
				!$this->registerHook('rightColumn') or
				!$this->registerHook('leftColumn') or
				!$this->registerHook('adminCustomers'))
		return false;
		
		Configuration::updateValue('PRO_CUSTOMER_LINK', _MODULE_DIR_.'pro_customer/contact-pro.php');
		Configuration::updateValue('CUSTREG_IS_REGISTERED_BY_DEFAULT', 1);		
		Configuration::updateValue('CUSTREG_SEND_ADMIN_MAIL', 1);
		Configuration::updateValue('CUSTREG_MAILS', Configuration::get('PS_SHOP_EMAIL'));
		Configuration::updateValue('CUSTREG_GROUPS', '1');
		Configuration::updateValue('CUSTREG_ID_GROUPS', 1);
		Configuration::updateValue('CUSTREG_REGISTRATION_CODE', 0);
		Configuration::updateValue('CUSTREG_ASK_PRIVILEGE_ACCESS', 1);
		
		$tab = new Tab();
		$tab->id_parent = 2;
		$tab->name = array(Language::getIdByIso('en') =>'Pending',
		Language::getIdByIso('fr')  =>'En attente');
		$tab->class_name = 'AdminProCustomer';
		$tab->module = 'pro_customer';
		$tab->add();
		
		@copy(_PS_MODULE_DIR_.$tab->module.'/logo.gif', _PS_IMG_DIR_.'t/'.$tab->class_name.'.gif');
		
		$query = 'CREATE TABLE '._DB_PREFIX_.'pro_customer (
				`id_customer` int(2) NOT NULL, 
				`is_approved` int(2) NOT NULL, 
				`privilege_request` int(2) NOT NULL,
				`company` varchar(32),
				`number_type` int(2) NOT NULL, 
				`privilege_message` varchar(32),
				`send_email_approved` int(2) NOT NULL,
				`send_email_request` int(2) NOT NULL,				
				`date_add` datetime NOT NULL,
				`date_upd` datetime NOT NULL,
				PRIMARY KEY  (`id_customer`)) 
			ENGINE=MyISAM default CHARSET=utf8';
		if (!Db::getInstance()->Execute($query))
		return false;

		return true;
	}

	public function uninstall()
	{
		require_once (dirname(__FILE__).'/ClassProCustomer.php');

		$allNotApproved = ClassProCustomer::getAllNotApproved();
		foreach ($allNotApproved as $approvalLine) {
			$customer = new Customer($approvalLine['id_customer']);
			$customer->delete();
		}
		
		Configuration::deleteByName('CUSTREG_SEND_ADMIN_MAIL');
		Configuration::deleteByName('CUSTREG_MAILS');
		Configuration::deleteByName('CUSTREG_IS_REGISTERED_BY_DEFAULT');
		Configuration::deleteByName('CUSTREG_GROUPS');
		Configuration::deleteByName('CUSTREG_ID_GROUPS');
		
		$tab = new Tab(Tab::getIdFromClassName ('AdminProCustomer'));		
		if (! $tab->delete())
		return false;
		
		if (!parent::uninstall())
		return false;
		return Db::getInstance()->Execute('DROP TABLE '._DB_PREFIX_.'pro_customer');
	}

	function hookAuthentication()
	{
		require_once (dirname(__FILE__).'/ClassProCustomer.php');

		global $cookie;
		
		$approved = ClassProCustomer::isApproved ($cookie->id_customer);
		if (! $approved) {
			$cookie->logged = 0;
			$_POST['back'] = 'modules/'.basename(__FILE__, '.php').'/messages.php?msg=wait&back=my-account.php';
		}
		
	}

	function hookCreateAccountForm($params)
	{
		global $smarty, $link;
		$errors = array();
		$confirm = false;

		$smarty->assign(array(
		'isRegistrationCodeManaged' => (($this->_registrationCode != '') ? true : false),
		'isPrivilegedManaged' => (($this->_askPrivilegeAccess == 1) ? true : false)
		));
		return $this->display(__FILE__, 'ProCustomerForm.tpl');
	}
	
	
	function hookCreateAccount($params)
	{
		require_once (dirname(__FILE__).'/ClassProCustomer.php');

		global $cookie, $back;
		
		$registration_code = pSQL(Tools::getValue('registration_code'));
		$privilege_message = pSQL(Tools::getValue('privilege_message'));
		$privilege_request_check = pSQL(Tools::getValue('privilege_request_check'));
		$number_type = pSQL(Tools::getValue('number_type'));
		$company = Tools::getValue('company');
		
		if ($privilege_request_check == 1) {
			
			/* auto validate */
			if (isset($privilege_message) AND  $privilege_message <> '') {
				switch ($number_type) {
				case 0: /* siret number */
					$approvalSiret = ClassProCustomer::isSiret(ClassProCustomer::onlyNumeric($privilege_message));
					break;
				case 1: /* siren number */
					$approvalSiret = ClassProCustomer::isSiren(ClassProCustomer::onlyNumeric($privilege_message));
					break;
				case 2: /* vat number */
					$approvalSiret = ClassProCustomer::isVatNumber($privilege_message);
				default:
				}
			}		

			if ($this->_registrationCode == 1 AND $approvalSiret == 0 AND $this->_askPrivilegeAccess == 1 AND $company != '') {
				$customer = $params['newCustomer'];
				$customer->cleanGroups();
				$customer->addGroups ($this->_defaultGroups);			
			}
		}
		$cookie->logged = $this->_isCustomerRegistredByDefault;
		$cust = $params['newCustomer'];
		$approval = new ClassProCustomer($cust->id);
		$approval->is_approved = $this->_isCustomerRegistredByDefault;
		$approval->number_type = pSQL(Tools::getValue('number_type'));
		$approval->privilege_request = 0;
		$approval->company = pSQL(Tools::getValue('company'));
		
		if (empty($company))
			$errors[] = Tools::displayError($this->l('Company is required'));
		elseif (empty($privilege_message))
			$errors[] = Tools::displayError($this->l('ID is required'));
		
		if (!$errors) {
		if (($this->_sendAdminMail AND $privilege_request_check == 1) OR ($this->_sendAdminMail AND !$approval->is_approved)) {
			$approval->privilege_request = $privilege_request_check;
			$this->sendMail('pending_registration', array('customer' => $cust, 'approval'=>$approval));
		}
		if ($privilege_request_check == 1) {

			if ($this->_registrationCode == 1 AND $approvalSiret == 0 AND $this->_askPrivilegeAccess == 1 AND $company != '') {
				$approval->privilege_request = 2;
				$this->sendMail('customer_privilege_ok', array('id_customer'=>$approval->id_customer));
				$approval->send_email_request = 1;
			} elseif ($this->_registrationCode == 1 AND $this->_askPrivilegeAccess == 1 AND ($approvalSiret != 0 OR $company == '')) {
				$approval->privilege_request = 3;
				$this->sendMail('customer_privilege_refused', array('id_customer'=>$approval->id_customer));
				$approval->send_email_request = 1;
			} elseif ($this->_registrationCode == 0) {
				$approval->privilege_request = 1;
			}
			$approval->privilege_message = pSQL(Tools::getValue('privilege_message'), '');
		}
		if (! $approval->save()) { 
			Tools::d('Unable to save approval information');
		}
		} else {
		if ($privilege_request_check == 1) {
		$this->sendMail('customer_privilege_refused', array('id_customer'=>$approval->id_customer));
		}
		}
		if (!$approval->is_approved) {
			$back = 'modules/'.basename(__FILE__, '.php').'/messages.php?msg=noconnect&back=my-account.php';
		} elseif ($back == '') {
			$back = 'my-account.php';
		}

	}

	function hookAdminCustomers($params)
	{
		require_once (dirname(__FILE__).'/ClassProCustomer.php');

		global $cookie;
		
		$numberType = array($this->l('SIRET Number'), $this->l('SIREN Number'), $this->l('VAT Number'));
		
		$id_customer = intval($params['id_customer']);
		$isApproved = ClassProCustomer::isApproved ($id_customer);
		$token = Tools::getAdminToken('AdminProCustomer'.intval(Tab::getIdFromClassName('AdminProCustomer')).intval($cookie->id_employee));
		$txt = '<p>'.$this->l('Customer registration').' : ';
		if ($isApproved) {
			$txt .= '<img src="../img/admin/enabled.gif" />';
		} else {
			$txt .= '<a href="?tab=AdminProCustomer&approvecustomer&id_customer='.$id_customer.'&token='.$token.
			'&back='.urlencode($_SERVER['REQUEST_URI']).'"><span class="button">'.$this->l('Validate').'</span></a>';
			$txt .= '<a href="?tab=AdminProCustomer&refusecustomer&id_customer='.$id_customer.'&token='.$token.
			'"><span class="button">'.$this->l('Refuse').'</span></a>';
		}
		$approvalInfos = ClassProCustomer::getPrivilegeRequest ($id_customer);
		$txt .= '</p><p>'.$this->l('Company').' : '.$approvalInfos['company'].'</p>'.$this->l('Pro access').' : ';
		
		switch ($approvalInfos['privilege_request']) {
		case 0: /* no request */
			$txt .= $this->l('Not requested');
			break;
		case 1: /* requested */
			//$txt .=  '<a href="?tab=AdminProCustomer&id_customer='.$id_customer.'&approveprivilegecustomer&token='.$token.'&back='.urlencode($_SERVER['REQUEST_URI']).'" ><img src="../img/admin/enabled.gif"></a>';
			//$txt .=  '<a href="?tab=AdminProCustomer&id_customer='.$id_customer.'&refuseprivilegecustomer&token='.$token.'&back='.urlencode($_SERVER['REQUEST_URI']).'" ><img src="../img/admin/disabled.gif"></a>';
			$txt .=  $this->l('Awaiting Validation');
			break;
		case 2: /* granted */
			$txt .= '<img src="../img/admin/enabled.gif" />';
			break;
		case 3: /* refused */
			$txt .= '<img src="../img/admin/disabled.gif" />';
			break;
		default: /* unknown */
			$txt .=  $this->l('Not requested');
		}
		
		if (isset($approvalInfos['privilege_message']) AND  $approvalInfos['privilege_message'] <> '') {
			switch ($approvalInfos['number_type']) {
			case 0: /* siret number */
				$approvalSiret = ClassProCustomer::isSiret(ClassProCustomer::onlyNumeric($approvalInfos['privilege_message']));
				break;
			case 1: /* siren number */
				$approvalSiret = ClassProCustomer::isSiren(ClassProCustomer::onlyNumeric($approvalInfos['privilege_message']));
				break;
			case 2: /* vat number */
				$approvalSiret = ClassProCustomer::isVatNumber($approvalInfos['privilege_message']);
			default:
			}	
			$txt .= '<p>'.$numberType[$approvalInfos['number_type']].' : '.$approvalInfos['privilege_message'].' '.($approvalSiret == 0 ? '<img src="../img/admin/enabled.gif" />' : '<img src="../img/admin/disabled.gif" />').'</p>';
		}
		$txt .= '</p>';
		
		return '<h2>'.$this->l('Registration').'</h2><fieldset style="width: 400px"><div style="float: right"><a href="?tab=AdminProCustomer&addcustomer&id_customer='.$id_customer.'&token='.$token.'"><img src="../img/admin/edit.gif" /></a></div>
				'.$txt . '
				</fieldset>
				<div class="clear">&nbsp;</div>';
	}


	function hookBackOfficeHome()
	{
		require_once (dirname(__FILE__).'/ClassProCustomer.php');

		global $cookie;
		
		$txt = '';
		if ($this->_isCustomerRegistredByDefault == 0) {
			$allNotApproved = ClassProCustomer::getAllNotApproved();
			$nbCustomers = sizeof ($allNotApproved);
			$txt .= '<div class="clear space"></div><fieldset class="width3"><legend>'.
			$this->l('Pending customer registration').'</legend>';
			$txt .= $this->l('Number of customers waiting for registration : ');
		} else {
			$allNotProAccess = ClassProCustomer::getAllNotProAccess();
			$nbCustomers = sizeof ($allNotProAccess);
			$txt .= '<div class="clear space"></div><fieldset class="width3"><legend>'.
			$this->l('Pending customer Pro access').'</legend>';
			$txt .= $this->l('Number of customers waiting for Pro validation : ');
		}	
		if ($nbCustomers == 0)	
		$txt .= '&nbsp;<b>'.$this->l('None').'</b>';
		else 
		$txt .= '&nbsp;<b>'.$nbCustomers.'</b>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href="?tab=AdminProCustomer&configure=pro_customer&token='.Tools::getAdminToken('AdminProCustomer'.intval(Tab::getIdFromClassName('AdminProCustomer')).intval($cookie->id_employee)).'"><span  class="button">'.$this->l('View').'</span></a>';
		$txt .= '</fieldset><br/>';
		
		return $txt;
		
	}
	
	private function _displayForm()
	{
		global $cookie;
		/* Languages preliminaries */
		$defaultLanguage = intval(Configuration::get('PS_LANG_DEFAULT'));
		$languages = Language::getLanguages();
		$iso = Language::getIsoById(intval($cookie->id_lang));
		$divLangName = 'cpara';

		/* xml loading */
		$xml = false;
		if (file_exists(dirname(__FILE__).'/editorialpro.xml'))
		if (!$xml = simplexml_load_file(dirname(__FILE__).'/editorialpro.xml'))
		$this->_html .= $this->displayError($this->l('Your editor file is empty.'));
		
		// TinyMCE
		global $cookie;
		$iso = Language::getIsoById((int)($cookie->id_lang));
		$isoTinyMCE = (file_exists(_PS_ROOT_DIR_.'/js/tiny_mce/langs/'.$iso.'.js') ? $iso : 'en');
		$ad = dirname($_SERVER["PHP_SELF"]);
		$this->_html .=  '
			<script type="text/javascript">	
			var iso = \''.$isoTinyMCE.'\' ;
			var pathCSS = \''._THEME_CSS_DIR_.'\' ;
			var ad = \''.$ad.'\' ;
			</script>
			<script type="text/javascript" src="'.__PS_BASE_URI__.'js/tiny_mce/tiny_mce.js"></script>
			<script type="text/javascript" src="'.__PS_BASE_URI__.'js/tinymce.inc.js"></script>';
		$this->_html .= '
		<script type="text/javascript">id_language = Number('.$defaultLanguage.');</script>
				<label>'.$this->l('Introductory text').'</label>
				<div class="margin-form">';

				foreach ($languages as $language)
				{
					$this->_html .= '
					<div id="cpara_'.$language['id_lang'].'" style="display: '.($language['id_lang'] == $defaultLanguage ? 'block' : 'none').';float: left;">
						<textarea class="rte" cols="70" rows="30" id="body_paragraph_'.$language['id_lang'].'" name="body_paragraph_'.$language['id_lang'].'">'.($xml ? stripslashes(htmlspecialchars($xml->body->{'paragraph_'.$language['id_lang']})) : '').'</textarea>
					</div>';
				 }
				
				$this->_html .= $this->displayFlags($languages, $defaultLanguage, $divLangName, 'cpara', true);
				
				$this->_html .= '
					<p class="clear">'.$this->l('Text of your choice.').'</p>
				</div>';
	}
	
	private function _displayFormOld()
	{
		global $cookie;
		/* Languages preliminaries */
		$defaultLanguage = intval(Configuration::get('PS_LANG_DEFAULT'));
		$languages = Language::getLanguages();
		$iso = Language::getIsoById(intval($cookie->id_lang));
		$divLangName = 'cpara';

		/* xml loading */
		$xml = false;
		if (file_exists(dirname(__FILE__).'/editorialpro.xml'))
		if (!$xml = simplexml_load_file(dirname(__FILE__).'/editorialpro.xml'))
		$this->_html .= $this->displayError($this->l('Your editor file is empty.'));

		$this->_html .= '
		<script type="text/javascript" src="'.__PS_BASE_URI__.'js/tinymce/jscripts/tiny_mce/tiny_mce.js"></script>
		<script type="text/javascript">
					tinyMCE.init({
						mode : "textareas",
						theme : "advanced",
						plugins : "safari,pagebreak,style,layer,table,advimage,advlink,inlinepopups,media,searchreplace,contextmenu,paste,directionality,fullscreen",
						// Theme options
						theme_advanced_buttons1 : "newdocument,|,bold,italic,underline,strikethrough,|,justifyleft,justifycenter,justifyright,justifyfull,styleselect,formatselect,fontselect,fontsizeselect",
						theme_advanced_buttons2 : "cut,copy,paste,pastetext,pasteword,|,search,replace,|,bullist,numlist,|,outdent,indent,blockquote,|,undo,redo,|,link,unlink,anchor,image,cleanup,help,code,,|,forecolor,backcolor",
						theme_advanced_buttons3 : "tablecontrols,|,hr,removeformat,visualaid,|,sub,sup,|,charmap,media,|,ltr,rtl,|,fullscreen",
						theme_advanced_buttons4 : "insertlayer,moveforward,movebackward,absolute,|,styleprops,|,cite,abbr,acronym,del,ins,attribs,|,pagebreak",
						theme_advanced_toolbar_location : "top",
						theme_advanced_toolbar_align : "left",
						theme_advanced_statusbar_location : "bottom",
						theme_advanced_resizing : false,
						content_css : "'.__PS_BASE_URI__.'themes/'._THEME_NAME_.'/css/global.css",
						document_base_url : "'.__PS_BASE_URI__.'",
						width: "600",
						height: "auto",
						font_size_style_values : "8pt, 10pt, 12pt, 14pt, 18pt, 24pt, 36pt",
						// Drop lists for link/image/media/template dialogs
						template_external_list_url : "lists/template_list.js",
						external_link_list_url : "lists/link_list.js",
						external_image_list_url : "lists/image_list.js",
						media_external_list_url : "lists/media_list.js",
						elements : "nourlconvert",
						entity_encoding: "raw",
						convert_urls : false,
						language : "'.(file_exists(_PS_ROOT_DIR_.'/js/tinymce/jscripts/tiny_mce/langs/'.$iso.'.js') ? $iso : 'en').'"
						
					});
		</script>
		<script type="text/javascript">id_language = Number('.$defaultLanguage.');</script>';
		$this->_html .= '
				<label>'.$this->l('Introductory text').'</label>
				<div class="margin-form">';

		foreach ($languages as $language)
		{
			$this->_html .= '
					<div id="cpara_'.$language['id_lang'].'" style="display: '.($language['id_lang'] == $defaultLanguage ? 'block' : 'none').';float: left;">
						<textarea class="rte" cols="70" rows="30" id="body_paragraph_'.$language['id_lang'].'" name="body_paragraph_'.$language['id_lang'].'">'.($xml ? stripslashes(htmlspecialchars($xml->body->{'paragraph_'.$language['id_lang']})) : '').'</textarea>
					</div>';
		}
		
		$this->_html .= $this->displayFlags($languages, $defaultLanguage, $divLangName, 'cpara', true);
		
		$this->_html .= '
					<p class="clear">'.$this->l('Text of your choice.').'</p>
				</div>';
	}		
	
	function _display_config ()	
	{
		global $cookie;
		require_once (dirname(__FILE__).'/ClassProCustomer.php');

		$this->_html .=  '<form action="'.$_SERVER['REQUEST_URI'].'" method="post" enctype="multipart/form-data"><fieldset><legend><img src="'.$this->_path.'/logo.gif" alt="" title="" />'.$this->l('Registration Settings').'</legend>';
		$this->_html .=  '
		<p>'.$this->l('You can choose if customers are registrered by default or blocked until admin validation.').'<br>'.
		$this->l('Depending of the option, use appropriate mail template to be sent to customers.').'</p>
		<label>'.$this->l('Registered by default :').'</label>
			<div class="margin-form">
					<input type="radio" name="is_default_registered" id="is_default_registered_on" value="1" '.(($this->_isCustomerRegistredByDefault) ? 'checked="checked" ' : '').'/>
					<label class="t" for="is_default_registered_on"> <img src="../img/admin/enabled.gif" alt="'.$this->l('Yes').'" title="'.$this->l('Yes').'" /></label>
					<input type="radio" name="is_default_registered" id="is_default_registered_off" value="0" '.((!$this->_isCustomerRegistredByDefault) ? 'checked="checked" ' : '').'/>
					<label class="t" for="is_default_registered_off"> <img src="../img/admin/disabled.gif" alt="'.$this->l('No').'" title="'.$this->l('No').'" /></label>
			</div>
			<br />
			<p>'.$this->l('An email can be sent to administrator on new pending customer registration').'</p>
			<label>'.$this->l('Send e-mail :').'</label>
			<div class="margin-form">
					<input type="radio" name="mail_active" id="mail_active_on" value="1" '.(($this->_sendAdminMail) ? 'checked="checked" ' : '').'/>
					<label class="t" for="mail_active_on"> <img src="../img/admin/enabled.gif" alt="'.$this->l('Yes').'" title="'.$this->l('Yes').'" /></label>
					<input type="radio" name="mail_active" id="mail_active_off" value="0" '.((!$this->_sendAdminMail) ? 'checked="checked" ' : '').'/>
					<label class="t" for="mail_active_off"> <img src="../img/admin/disabled.gif" alt="'.$this->l('No').'" title="'.$this->l('No').'" /></label>
			</div>
			<label>'.$this->l('E-mail adresses :').'</label>
			<div class="margin-form">
					<input id="mails" type="text" size="64" name="mails" value="'.str_replace(",", ";", $this->_mails).'" />
					<p>'.$this->l('Use the semicolon (;) to separate entries.').'</p>
			</div>
			<div class="margin-form">
					<input type="submit" value="'.$this->l('Update').'" name="submitUpdate" class="button" />
			</div>
			</fieldset>';

		$this->_html .= '
		<br/>
		<fieldset><legend><img src="'.$this->_path.'/logo.gif" alt="" title="" />'.$this->l('Pro Access Settings').'</legend>';
		$this->_html .= '<p>'.
		$this->l('Management of access according to the pro group')
		.'</p>';
		$this->_html .= '<label>'.$this->l('Pro access request :').'</label>
				<div class="margin-form">
					<input type="radio" name="ask_privilege_access" id="ask_privilege_access_on" value="1" '.(($this->_askPrivilegeAccess) ? 'checked="checked" ' : '').'/>
					<label class="t" for="ask_privilege_access_on"> <img src="../img/admin/enabled.gif" alt="'.$this->l('Yes').'" title="'.$this->l('Yes').'" /></label>
					<input type="radio" name="ask_privilege_access" id="ask_privilege_access_off" value="0" '.((!$this->_askPrivilegeAccess) ? 'checked="checked" ' : '').'/>
					<label class="t" for="ask_privilege_access_off"> <img src="../img/admin/disabled.gif" alt="'.$this->l('No').'" title="'.$this->l('No').'" /></label>
				</div>';

		$this->_html .= '<br/><label>'.$this->l('Groups :').' </label>
				<div class="margin-form">';
		$groups = Group::getGroups($cookie->id_lang, true);

		if (sizeof($groups))
		{
			$this->_html .=  '
					<table cellspacing="0" cellpadding="0" class="table" style="width: 29.5em;">
						<tr>
							<th><input type="checkbox" name="checkme" class="noborder" onclick="checkDelBoxes(this.form, \'groupBox[]\', this.checked)" /></th>
							<th>'.$this->l('ID').'</th>
							<th>'.$this->l('Group name').'</th>
						</tr>';
			$irow = 0;
			foreach ($groups as $group)
			{   
				$this->_html .=  '
							<tr class="'.($irow++ % 2 ? 'alt_row' : '').'">
								<td>'.($group['id_group'] != 0 ? '<input type="checkbox" name="groupBox[]" class="groupBox" id="groupBox_'.$group['id_group'].'" value="'.$group['id_group'].'" '.(in_array($group['id_group'], $this->_defaultGroups) ? 'checked="checked" ' : '').'/>' : '').'</td>
								<td>'.$group['id_group'].'</td>
								<td><label for="groupBox_'.$group['id_group'].'" class="t">'.$group['name'].'</label></td>
							</tr>';
			}
			$this->_html .=  '
					</table>
					<p style="padding:0px; margin:10px 0px 10px 0px;">'.$this->l('Mark all checkbox(es) of groups to which the customer is to be member').'</p>
					';
		} else
		$this->_html .=  '<p>'.$this->l('No group created').'</p>';
		$this->_html .= '</div><label>'.$this->l('Default group :').'</label>
				<div class="margin-form">
					<select name="id_default_group" onchange="checkDefaultGroup(this.value);">';
				foreach ($groups as $group)
					$this->_html .= '<option value="'.intval($group['id_group']).'"'.($group['id_group'] == $this->_idDefaultGroups ? ' selected="selected"' : '').'>'.htmlentities($group['name'], ENT_NOQUOTES, 'utf-8').'</option>';
					$this->_html .= '</select><p>'.$this->l('The group from which apply none cumulative rules (e.g., price display method, reduction)').'</p>';
		$this->_html .=  '</div><br/>';		
		$this->_html .= '<label>'.$this->l('Auto valid by ID number valid :').'</label>
				<div class="margin-form">
					<input type="radio" name="registration_code" id="registration_code_on" value="1" '.(($this->_registrationCode) ? 'checked="checked" ' : '').'/>
					<label class="t" for="registration_code_on"> <img src="../img/admin/enabled.gif" alt="'.$this->l('Yes').'" title="'.$this->l('Yes').'" /></label>
					<input type="radio" name="registration_code" id="registration_code_off" value="0" '.((!$this->_registrationCode) ? 'checked="checked" ' : '').'/>
					<label class="t" for="registration_code_off"> <img src="../img/admin/disabled.gif" alt="'.$this->l('No').'" title="'.$this->l('No').'" /></label>
				</div><br/>
			<div class="margin-form">
					<input type="submit" value="'.$this->l('Update').'" name="submitUpdate" class="button" />
			</div>
			</fieldset>
			<br>';
		$this->_html .= '
			<fieldset><legend><img src="'.$this->_path.'/logo.gif" alt="" title="" />'.$this->l('Image block configuration').'</legend>
			<a href="'.$this->pro_link.'" target="_blank" title="'.$this->l('Link').'">';
		if ($this->pro_img)
		$this->_html .= '<img src="'.$this->pro_img.'" alt="'.$this->l('Image').'" style="margin-left: 100px;"/>';
		else
		$this->_html .= $this->l('no image');
		$this->_html .= '
		</a>
		<br/>
		<br/>
		<label for="pro_img">'.$this->l('Change image').'&nbsp;&nbsp;</label><input id="pro_img" type="file" name="pro_img" />
		<br/>
		<br class="clear"/>
		<label for="pro_link">'.$this->l('Image link').'&nbsp;&nbsp;</label><input id="pro_link" type="text" size="64" name="pro_link" value="'.$this->pro_link.'" />
		<br/>
		<br class="clear"/>';
		/* display the editorial's form */
		if (version_compare(_PS_VERSION_, '1.4.0') >= 0) {
		$this->_displayForm();
		} else {
		$this->_displayFormOld();
		}
		$this->_html .= '
		<br/>
					<div class="margin-form">
					<input type="submit" value="'.$this->l('Update').'" name="submitUpdate" class="button" />
			</div>
		</fieldset>
		</form>
	';
		$this->_html .='<div>&nbsp;</div><fieldset>
					<legend>'.$this->l('About').'</legend>
					<p style="font-size: 1.5em; font-weight: bold; padding-bottom: 0"><img src="'.$this->_path.'logo.gif" alt="'.$this->displayName.'" style="float: left; padding-right: 1em"/>'.$this->displayName.'</p>
					<p style="clear: both">
					'.$this->description.'
					</p>
					<p>
					'.$this->l('Developped by').' <a style="color: #7ba45b; text-decoration: underline;" href="http://dpfpic.com" target="_blank">'.$this->author.'</a> '.$this->date.'.
					</p>
				</fieldset><div>&nbsp;</div>';		

	}
	
	function displayFrontForm()
	{		
		require_once (dirname(__FILE__).'/ClassProCustomer.php');
		
		global $smarty, $cookie;
		$errors = array();
		$confirm = false;

		if (isset($_POST['submitProFrom']))
		{
			$passwd = ($cookie->passwd ? $cookie->passwd : trim(Tools::getValue('passwd')));
			$email = ($cookie->email ? $cookie->email : trim(Tools::getValue('email')));
			$company = trim(Tools::getValue('company'));
			$privilege_message = trim(Tools::getValue('privilege_message'));
			$number_type = pSQL(Tools::getValue('number_type'));

			if (!$cookie->logged AND empty($email))
			$errors[] = Tools::displayError($this->l('e-mail address is required'));
			elseif (!$cookie->logged AND !Validate::isEmail($email))
			$errors[] = Tools::displayError($this->l('invalid e-mail address'));
			elseif (!$cookie->logged AND empty($passwd))
			$errors[] = Tools::displayError($this->l('password is required'));
			elseif (!$cookie->logged AND Tools::strlen($passwd) > 32)
			$errors[] = Tools::displayError($this->l('password is too long'));
			elseif (!$cookie->logged AND !Validate::isPasswd($passwd))
			$errors[] = Tools::displayError($this->l('invalid password'));
			elseif (empty($company))
			$errors[] = Tools::displayError($this->l('Company is required'));
			elseif (empty($privilege_message))
			$errors[] = Tools::displayError($this->l('ID is required'));		
			else
			{
				$customer = new Customer($cookie->id_customer);
				$authentication = $customer->getByEmail(trim($email), trim($passwd));
				$id_customer = intval($customer->id);

				/* Handle brute force attacks */
				sleep(1);
				if (!$cookie->logged AND (!$authentication OR !$customer->id))
				$errors[] = Tools::displayError($this->l('authentication failed'));
				else
				{
					
					/* auto validate */
					switch ($number_type) {
					case 0: /* siret number */
						$approvalSiret = ClassProCustomer::isSiret(ClassProCustomer::onlyNumeric($privilege_message));
						break;
					case 1: /* siren number */
						$approvalSiret = ClassProCustomer::isSiren(ClassProCustomer::onlyNumeric($privilege_message));
						break;
					case 2: /* vat number */
						$approvalSiret = ClassProCustomer::isVatNumber($privilege_message);
					default:
					}

					$approval = new ClassProCustomer($id_customer);
					if ($approval->privilege_request != 2) {
						$approval->is_approved = 1;
						$approval->send_email_approved = 1;
						$approval->number_type = $number_type;
						$approval->privilege_request = 1;
						$approval->company = $company;
						$approval->privilege_message = $privilege_message;
						
						if ($this->_sendAdminMail) {
							$approval->privilege_request = 1;
							$this->sendMail('pending_registration', array('customer' => $customer, 'approval'=>$approval));
						}
						if ($this->_registrationCode == 1 AND $approvalSiret == 0 AND $company != '') {
							$customer->cleanGroups();
							$customer->addGroups ($this->_defaultGroups);			
						}
						if ($this->_registrationCode == 1 AND $approvalSiret == 0 AND $company != '') {
							$approval->privilege_request = 2;
							$this->sendMail('customer_privilege_ok', array('id_customer'=>$approval->id_customer));
							$approval->send_email_request = 1;
						} elseif ($this->_registrationCode == 1 AND ($approvalSiret != 0 OR $company == '')) {
							$approval->privilege_request = 3;
							$this->sendMail('customer_privilege_refused', array('id_customer'=>$approval->id_customer));
							$approval->send_email_request = 1;
						} elseif ($this->_registrationCode == 0) {
							$approval->privilege_request = 1;
						}
						
						$confirm = $this->l('Your request was successfully sent !');
						
						if (! $approval->save()) { 
							Tools::d('Unable to save approval information');
						}
					} else {
						$confirm = $this->l('You have already requested access !');
					}
					
				}
			}	
		}
		if (file_exists(dirname(__FILE__).'/editorialpro.xml'))
		{
			if ($xml = simplexml_load_file(dirname(__FILE__).'/editorialpro.xml'))
			{
				$smarty->assign(array(
				'xml' => $xml,
				'paragraph' => 'paragraph_'.$cookie->id_lang
				));
				
			}
		}
		
		$smarty->assign(array(
		'validate_login' => $cookie->logged,
		'errors' => $errors,
		'confirm' => $confirm
		));
		return $this->display(__FILE__, 'contactprocustomer.tpl');
	}
	
	function putContent($xml_data, $key, $field, $forbidden, $section)
	{
		foreach ($forbidden AS $line)
		if ($key == $line)
		return 0;
		if (!preg_match('/^'.$section.'_/i', $key))
		return 0;
		$key = preg_replace('/^'.$section.'_/i', '', $key);
		$field = htmlspecialchars($field);
		if (!$field)
		return 0;
		return ("\n".'		<'.$key.'>'.$field.'</'.$key.'>');
	}
	
	function getContent ()	
	{
		require_once (dirname(__FILE__).'/ClassProCustomer.php');

		global $cookie;

		/* display the module name */
		$this->_html = '<h2>'.$this->displayName.'</h2>';
		
		if (!empty($_POST))
		{
			$this->_postValidation();
			if (!$this->_postErrors)
			$this->_postProcess();
		}
		
		$this->_display_config ();
		

		/* Table clean-up : remove all entries for which the customer has already been deleted from customer table */
		Db::getInstance()->Execute('DELETE FROM `'._DB_PREFIX_.'pro_customer` WHERE `id_customer` not in (SELECT a.id_customer from `'._DB_PREFIX_.'customer` a)');

		return $this->_html; 
	}

	
	
	private function _postProcess()
	{
		global $currentIndex;
		
		$this->_isCustomerRegistredByDefault = intval(Tools::getValue('is_default_registered', false));
		Configuration::updateValue('CUSTREG_IS_REGISTERED_BY_DEFAULT', $this->_isCustomerRegistredByDefault);		
		
		$this->_sendAdminMail = intval(Tools::getValue('mail_active'));
		Configuration::updateValue('CUSTREG_SEND_ADMIN_MAIL', $this->_sendAdminMail);		
		
		$this->_askPrivilegeAccess = intval(Tools::getValue('ask_privilege_access'));
		Configuration::updateValue('CUSTREG_ASK_PRIVILEGE_ACCESS', $this->_askPrivilegeAccess);		
		
		$this->_registrationCode = pSQL(Tools::getValue('registration_code'));
		Configuration::updateValue('CUSTREG_REGISTRATION_CODE', $this->_registrationCode);		
		$mails = explode(";", $_POST['mails']);
		$this->_mails = '';
		foreach ($mails as $mail)
		$this->_mails .= trim($mail).self::__MAIL_DELIMITOR__;
		$this->_mails = trim($this->_mails, self::__MAIL_DELIMITOR__);
		Configuration::updateValue('CUSTREG_MAILS', $this->_mails);
		
		$this->_defaultGroups = Tools::getValue('groupBox');
		if (is_array($this->_defaultGroups))
		$this->_defaultGroupsString = '1;'.implode(';', $this->_defaultGroups);
		else
		$this->_defaultGroupsString = '1';
		Configuration::updateValue('CUSTREG_GROUPS', $this->_defaultGroupsString);
		$this->_idDefaultGroups = intval(Tools::getValue('id_default_group'));
		Configuration::updateValue('CUSTREG_ID_GROUPS', $this->_idDefaultGroups);
		
	}

	private function _postValidation()
	{
		$groupList = Tools::getValue('groupBox');
		$this->_postErrors = false;
		if (Tools::isSubmit('submitUpdate'))
		{
			$file = false;
			if (isset($_FILES['pro_img']) AND isset($_FILES['pro_img']['tmp_name']) AND !empty($_FILES['pro_img']['tmp_name']))
			{
				if ($error = checkImage($_FILES['pro_img'], 4000000))
				$this->_postErrors .= $error;
				elseif (!move_uploaded_file($_FILES['pro_img']['tmp_name'], dirname(__FILE__).'/img/'.$this->pro_imgname))
				$this->_postErrors .= $this->l('Error move uploaded file');

				$this->pro_img = $this->_path.'/img/'.$this->pro_imgname;
			}
			if ($link = Tools::getValue('pro_link'))
			{
				Configuration::updateValue('PRO_CUSTOMER_LINK', $link);
				$this->pro_link = htmlentities($link, ENT_QUOTES, 'UTF-8');
			}
			// Forbidden key
			$forbidden = array('submitUpdate');
			
			/*foreach ($_POST AS $key => $value)
			if (!Validate::isCleanHtml($_POST[$key]))
			{
				$this->_html .= $this->displayError($this->l('Invalid html field, javascript is forbidden'));
				$this->_displayForm();
				return $this->_html;
			}*/

			// Generate new XML data
			$newXml = '<?xml version=\'1.0\' encoding=\'utf-8\' ?>'."\n";
			$newXml .= '<editorial>'."\n";
			$newXml .= '	<header>';
			// Making header data
			foreach ($_POST AS $key => $field)
			if ($line = $this->putContent($newXml, $key, $field, $forbidden, 'header'))
			$newXml .= $line;
			$newXml .= "\n".'	</header>'."\n";
			$newXml .= '	<body>';
			// Making body data
			foreach ($_POST AS $key => $field)
			if ($line = $this->putContent($newXml, $key, $field, $forbidden, 'body'))
			$newXml .= $line;
			$newXml .= "\n".'	</body>'."\n";
			$newXml .= '</editorial>'."\n";

			/* write it into the editorial xml file */
			if ($fd = @fopen(dirname(__FILE__).'/editorialpro.xml', 'w'))
			{
				if (!@fwrite($fd, $newXml))
				$this->_postErrors .= $this->displayError($this->l('Unable to write to the editor file.'));
				if (!@fclose($fd))
				$this->_postErrors .= $this->displayError($this->l('Can\'t close the editor file.'));
			}
			else
			$this->_postErrors .= $this->displayError($this->l('Unable to update the editor file.<br />Please check the editor file\'s writing permissions.'));

			if (!isset($_POST['mails']) OR empty($_POST['mails']))
			$this->_postErrors .= $this->l('No e-mail addresses specified');
			else
			{
				$mails = explode(";", $_POST['mails']);
				foreach ($mails as $mail)
				{
					$mail = trim($mail);
					if (!empty($mail) AND !Validate::isEmail($mail))
					$this->_postErrors  .= $this->l('Invalid e-mail: ').$mail.'.';
				}
			}
			if (!is_array($groupList) OR sizeof($groupList) == 0)
								$this->_postErrors  .= $this->l('customer must be in at least one group');
							else
								if (!in_array(Tools::getValue('id_default_group'), $groupList))
									$this->_postErrors  .= $this->l('default customer group must be selected on group box');
							
			
			$this->_html .= $this->_postErrors == '' ? $this->displayConfirmation($this->l('Settings updated successfully')) : '';
		}
		if ($this->_postErrors)
		echo $this->displayError($this->_postErrors);
	}

	
	public function sendMail ($mailName, $params)
	{
		require_once (dirname(__FILE__).'/ClassProCustomer.php');

		switch ($mailName) {
		case 'customer_registration_ok':
			$id_customer = intval($params['id_customer']);
			if ($id_customer <> 0) {
				$user = new Customer ($id_customer);
				Mail::Send(intval(Configuration::get('PS_LANG_DEFAULT')),
				'customer_registration_ok', 
				$this->l('Your account is activated'), 
				array('{firstname}' => $user->firstname, '{lastname}' => $user->lastname, '{email}' => $user->email, '{passwd}' => $this->l('Already sent')), 
				$user->email, NULL, strval(Configuration::get('PS_SHOP_EMAIL')), 
				strval(Configuration::get('PS_SHOP_NAME')), NULL, NULL, dirname(__FILE__).'/mails/');	
			}
			break;
			
		case 'pending_registration':
			$customer = $params['customer'];
			$approval = $params['approval'];
			$template = 'pending_registration';
			if ($approval->privilege_request)
			$subject = $this->l('New customer with Pro access waiting for registration');
			else
			$subject = $this->l('New customer waiting for registration');
			$templateVars = array(
			'{firstname}' => $customer->firstname,
			'{lastname}' => $customer->lastname,
			'{email}' => $customer->email,
			'{privilege_request}' => (($approval->privilege_request) ?$this->l('YES') :$this->l('NO') )
			);
			$dest = explode(self::__MAIL_DELIMITOR__, $this->_mails);
			if (!Mail::Send(intval(Configuration::get('PS_LANG_DEFAULT')), $template, $subject, $templateVars,
						$dest, 
						NULL, $configuration['PS_SHOP_EMAIL'], $configuration['PS_SHOP_NAME'], NULL, NULL, dirname(__FILE__).'/mails/') ) {
				//die('Send Mail Failed');
			}
			if ($approval->privilege_request == 1) {
				$template = 'customer_pending';
				$subject = $this->l('Pro access waiting for validation');
				if (!Mail::Send(intval(Configuration::get('PS_LANG_DEFAULT')), $template, $subject, $templateVars,
							$customer->email, 
							NULL, $configuration['PS_SHOP_EMAIL'], $configuration['PS_SHOP_NAME'], NULL, NULL, dirname(__FILE__).'/mails/') ) {
					//die('Send Mail Failed');
				}	
			}
			break;
			
		case 'customer_privilege_ok':
			$id_customer = intval($params['id_customer']);
			if ($id_customer <> 0) {
				$user = new Customer ($id_customer);
				Mail::Send(intval(Configuration::get('PS_LANG_DEFAULT')),
				'customer_privilege_ok', 
				$this->l('Pro access granted'), 
				array('{firstname}' => $user->firstname, '{lastname}' => $user->lastname, '{email}' => $user->email, '{passwd}' => $this->l('Already sent')), 
				$user->email, NULL, strval(Configuration::get('PS_SHOP_EMAIL')), 
				strval(Configuration::get('PS_SHOP_NAME')), NULL, NULL, dirname(__FILE__).'/mails/');	
			}
			break;
			
		case 'customer_privilege_refused':
			$id_customer = intval($params['id_customer']);
			if ($id_customer <> 0) {
				$user = new Customer ($id_customer);
				Mail::Send(intval(Configuration::get('PS_LANG_DEFAULT')),
				'customer_privilege_refused', 
				$this->l('Pro access refused'), 
				array('{firstname}' => $user->firstname, '{lastname}' => $user->lastname, '{email}' => $user->email, '{passwd}' => $this->l('Already sent')), 
				$user->email, NULL, strval(Configuration::get('PS_SHOP_EMAIL')), 
				strval(Configuration::get('PS_SHOP_NAME')), NULL, NULL, dirname(__FILE__).'/mails/');	
			}
			break;
			
			
		}

	}
	
	/**
	* Returns module content
	*
	* @param array $params Parameters
	* @return string Content
	*/
	function hookRightColumn($params)
	{
		global $smarty;

		$smarty->assign('pro_image', $this->pro_img);
		$smarty->assign('pro_link', $this->pro_link);

		return $this->display(__FILE__, 'blockprocustomer.tpl');
	}

	function hookLeftColumn($params)
	{
		return $this->hookRightColumn($params);
	}	
}


?>